@@ -9,7 +9,7 @@
  * in the Software without restriction, including without limitation the rights
  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following condition:
+ * furnished to do so, subject to the following conditions:
  * 
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
@@ -86,7 +86,7 @@ runopts * getrunopts(int argc, char ** argv) {
 					next = &opts->dsskeyfile;
 					break;
 				case 'r':
-					next = &opts->dsskeyfile;
+					next = &opts->rsakeyfile;
 					break;
 				case 'F':
 					opts->forkbg = 0;
