@@ -48,21 +48,21 @@
 int sessinitdone = 0;
 
 static void session_init(int sock, runopts *opts, int childpipe,
-		struct sockaddr *remote_addr);
+		struct sockaddr *remoteaddr);
 static void session_identification();
 static void checktimeouts();
 
 struct sshsession ses;
 
 void child_session(int sock, runopts *opts, int childpipe,
-		struct sockaddr *remote_addr) {
+		struct sockaddr *remoteaddr) {
 
 	fd_set readfd, writefd;
 	struct timeval timeout;
 	int val;
 	
 	crypto_init();
-	session_init(sock, opts, childpipe, remote_addr);
+	session_init(sock, opts, childpipe, remoteaddr);
 
 	/* exchange identification, version etc */
 	session_identification();
@@ -162,37 +162,35 @@ void checktimeouts() {
 }
 
 
-#ifdef DOCLEANUP
-/* clean up session vars etc - only usually called at exit(), but useful
- * for checking memory left lying about */
+/* clean up a session on exit */
 void session_cleanup() {
 	
 	TRACE(("enter session_cleanup"));
-	chancleanup();
-
+	
 	m_free(ses.session_id);
-	sign_key_free(ses.hostkey);
-	m_free(ses.remoteident);
-
-	buf_free(ses.readbuf);
-	buf_free(ses.decryptreadbuf);
-	buf_free(ses.payload);
-	buf_free(ses.writepayload);
+	if (ses.opts->hostkey) {
+		sign_key_free(ses.opts->hostkey);
+	}
 	m_free(ses.keys);
 
+	if (!sessinitdone) {
+		return;
+	}
+	chancleanup();
 
 	TRACE(("leave session_cleanup"));
 }
-#endif
 
 /* called only at the start of a session, set up initial state */
 static void session_init(int sock, runopts *opts, int childpipe,
-		struct sockaddr *remote_addr) {
+		struct sockaddr *remoteaddr) {
 
 	struct timeval tv;
 	TRACE(("enter session_init"));
 
-	ses.remote_addr = remote_addr;
+	ses.remoteaddr = remoteaddr;
+
+	ses.addrstring = getaddrstring(remoteaddr);
 
 	ses.sock = sock;
 	ses.maxfd = sock;
