@@ -9,7 +9,7 @@
  * in the Software without restriction, including without limitation the rights
  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following condition:
+ * furnished to do so, subject to the following conditions:
  * 
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
@@ -39,11 +39,11 @@
 #include "queue.h"
 #include "runopts.h"
 
-void child_session(int sock, runopts *opts, int childpipe,
-		struct sockaddr *remote_addr);
-#ifdef DOCLEANUP
+extern int sessinitdone;
+
 void session_cleanup();
-#endif
+void child_session(int sock, runopts *opts, int childpipe,
+		struct sockaddr *remoteaddr);
 
 struct key_context {
 
@@ -76,7 +76,8 @@ struct sshsession {
 	int childpipe; /* kept open until we successfully authenticate */
 	long connecttime; /* time of initial connection */
 
-	struct sockaddr *remote_addr;
+	struct sockaddr *remoteaddr; /* the host and port of the client */
+	unsigned char *addrstring; /* the text version of remoteaddr */
 
 	unsigned int maxfd; /* the maximum file descriptor to check with select() */
 	unsigned char *remoteident;
